"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from novu_py import models, utils
from novu_py._hooks import HookContext
from novu_py.novu_subscribers import NovuSubscribers
from novu_py.subscriptions import Subscriptions
from novu_py.types import BaseModel, OptionalNullable, UNSET
from novu_py.utils import get_security_from_env
from novu_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class Topics(BaseSDK):
    r"""Topics are a way to group subscribers together so that they can be notified of events at once. A topic is identified by a custom key. This can be helpful for things like sending out marketing emails or notifying users of new features. Topics can also be used to send notifications to the subscribers who have been grouped together based on their interests, location, activities and much more.
    https://docs.novu.co/subscribers/topics
    """

    subscriptions: Subscriptions
    subscribers: NovuSubscribers

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.subscriptions = Subscriptions(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.subscribers = NovuSubscribers(
            self.sdk_configuration, parent_ref=self.parent_ref
        )

    def list(
        self,
        *,
        request: Union[
            models.TopicsControllerListTopicsRequest,
            models.TopicsControllerListTopicsRequestTypedDict,
        ] = models.TopicsControllerListTopicsRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerListTopicsResponse:
        r"""List all topics

        This api returns a paginated list of topics.
        Topics can be filtered by **key**, **name**, or **includeCursor** to paginate through the list.
        Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.TopicsControllerListTopicsRequest)
        request = cast(models.TopicsControllerListTopicsRequest, request)

        req = self._build_request(
            method="GET",
            path="/v2/topics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_listTopics",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerListTopicsResponse(
                result=unmarshal_json_response(models.ListTopicsResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        request: Union[
            models.TopicsControllerListTopicsRequest,
            models.TopicsControllerListTopicsRequestTypedDict,
        ] = models.TopicsControllerListTopicsRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerListTopicsResponse:
        r"""List all topics

        This api returns a paginated list of topics.
        Topics can be filtered by **key**, **name**, or **includeCursor** to paginate through the list.
        Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.TopicsControllerListTopicsRequest)
        request = cast(models.TopicsControllerListTopicsRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/v2/topics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_listTopics",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerListTopicsResponse(
                result=unmarshal_json_response(models.ListTopicsResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        create_update_topic_request_dto: Union[
            models.CreateUpdateTopicRequestDto,
            models.CreateUpdateTopicRequestDtoTypedDict,
        ],
        fail_if_exists: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpsertTopicResponse:
        r"""Create a topic

        Creates a new topic if it does not exist, or updates an existing topic if it already exists. Use ?failIfExists=true to prevent updates.

        :param create_update_topic_request_dto:
        :param fail_if_exists: If true, the request will fail if a topic with the same key already exists
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpsertTopicRequest(
            fail_if_exists=fail_if_exists,
            idempotency_key=idempotency_key,
            create_update_topic_request_dto=utils.get_pydantic_model(
                create_update_topic_request_dto, models.CreateUpdateTopicRequestDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v2/topics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_update_topic_request_dto,
                False,
                False,
                "json",
                models.CreateUpdateTopicRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_upsertTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return models.TopicsControllerUpsertTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                models.TopicResponseDtoErrorData, http_res
            )
            raise models.TopicResponseDtoError(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        create_update_topic_request_dto: Union[
            models.CreateUpdateTopicRequestDto,
            models.CreateUpdateTopicRequestDtoTypedDict,
        ],
        fail_if_exists: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpsertTopicResponse:
        r"""Create a topic

        Creates a new topic if it does not exist, or updates an existing topic if it already exists. Use ?failIfExists=true to prevent updates.

        :param create_update_topic_request_dto:
        :param fail_if_exists: If true, the request will fail if a topic with the same key already exists
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpsertTopicRequest(
            fail_if_exists=fail_if_exists,
            idempotency_key=idempotency_key,
            create_update_topic_request_dto=utils.get_pydantic_model(
                create_update_topic_request_dto, models.CreateUpdateTopicRequestDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v2/topics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_update_topic_request_dto,
                False,
                False,
                "json",
                models.CreateUpdateTopicRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_upsertTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return models.TopicsControllerUpsertTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(
                models.TopicResponseDtoErrorData, http_res
            )
            raise models.TopicResponseDtoError(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        topic_key: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerGetTopicResponse:
        r"""Retrieve a topic

        Retrieve a topic by its unique key identifier **topicKey**

        :param topic_key: The key identifier of the topic
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerGetTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_getTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerGetTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        topic_key: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerGetTopicResponse:
        r"""Retrieve a topic

        Retrieve a topic by its unique key identifier **topicKey**

        :param topic_key: The key identifier of the topic
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerGetTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_getTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerGetTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        topic_key: str,
        update_topic_request_dto: Union[
            models.UpdateTopicRequestDto, models.UpdateTopicRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpdateTopicResponse:
        r"""Update a topic

        Update a topic name by its unique key identifier **topicKey**

        :param topic_key: The key identifier of the topic
        :param update_topic_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpdateTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            update_topic_request_dto=utils.get_pydantic_model(
                update_topic_request_dto, models.UpdateTopicRequestDto
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_topic_request_dto,
                False,
                False,
                "json",
                models.UpdateTopicRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_updateTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerUpdateTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        topic_key: str,
        update_topic_request_dto: Union[
            models.UpdateTopicRequestDto, models.UpdateTopicRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpdateTopicResponse:
        r"""Update a topic

        Update a topic name by its unique key identifier **topicKey**

        :param topic_key: The key identifier of the topic
        :param update_topic_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpdateTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            update_topic_request_dto=utils.get_pydantic_model(
                update_topic_request_dto, models.UpdateTopicRequestDto
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_topic_request_dto,
                False,
                False,
                "json",
                models.UpdateTopicRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_updateTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerUpdateTopicResponse(
                result=unmarshal_json_response(models.TopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        topic_key: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerDeleteTopicResponse:
        r"""Delete a topic

        Delete a topic by its unique key identifier **topicKey**.
        This action is irreversible and will remove all subscriptions to the topic.

        :param topic_key: The key identifier of the topic
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerDeleteTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="DELETE",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_deleteTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerDeleteTopicResponse(
                result=unmarshal_json_response(models.DeleteTopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        topic_key: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerDeleteTopicResponse:
        r"""Delete a topic

        Delete a topic by its unique key identifier **topicKey**.
        This action is irreversible and will remove all subscriptions to the topic.

        :param topic_key: The key identifier of the topic
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerDeleteTopicRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v2/topics/{topicKey}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_deleteTopic",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerDeleteTopicResponse(
                result=unmarshal_json_response(models.DeleteTopicResponseDto, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DigestTypeEnum(str, Enum):
    r"""The Digest Type"""

    REGULAR = "regular"
    BACKOFF = "backoff"
    TIMED = "timed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrdinalValueEnum(str, Enum):
    r"""Value of the ordinal"""

    DAY = "day"
    WEEKDAY = "weekday"
    WEEKEND = "weekend"
    SUNDAY = "sunday"
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .buttontypeenum import ButtonTypeEnum
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PayloadTypedDict(TypedDict):
    r"""Payload of the action result"""


class Payload(BaseModel):
    r"""Payload of the action result"""


class MessageActionResultTypedDict(TypedDict):
    payload: NotRequired[PayloadTypedDict]
    r"""Payload of the action result"""
    type: NotRequired[ButtonTypeEnum]
    r"""Type of button for the action result"""


class MessageActionResult(BaseModel):
    payload: Optional[Payload] = None
    r"""Payload of the action result"""

    type: Optional[ButtonTypeEnum] = None
    r"""Type of button for the action result"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventbody import EventBody, EventBodyTypedDict
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class WebhookResultDtoTypedDict(TypedDict):
    id: str
    r"""Unique identifier for the webhook result"""
    event: EventBodyTypedDict
    r"""Event body containing the webhook event data"""


class WebhookResultDto(BaseModel):
    id: str
    r"""Unique identifier for the webhook result"""

    event: EventBody
    r"""Event body containing the webhook event data"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WorkflowStatusEnum(str, Enum):
    r"""Status of the workflow"""

    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"
    ERROR = "ERROR"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class IntegrationTypedDict(TypedDict):
    r"""The updated configurations after auto-configuration"""


class Integration(BaseModel):
    r"""The updated configurations after auto-configuration"""


class AutoConfigureIntegrationResponseDtoTypedDict(TypedDict):
    success: bool
    r"""Indicates whether the auto-configuration was successful"""
    message: NotRequired[str]
    r"""Optional message describing the result or any errors that occurred"""
    integration: NotRequired[IntegrationTypedDict]
    r"""The updated configurations after auto-configuration"""


class AutoConfigureIntegrationResponseDto(BaseModel):
    success: bool
    r"""Indicates whether the auto-configuration was successful"""

    message: Optional[str] = None
    r"""Optional message describing the result or any errors that occurred"""

    integration: Optional[Integration] = None
    r"""The updated configurations after auto-configuration"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .redirectdto import RedirectDto, RedirectDtoTypedDict
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ActionDtoTypedDict(TypedDict):
    label: NotRequired[str]
    r"""Label for the action button."""
    redirect: NotRequired[RedirectDtoTypedDict]
    r"""Redirect configuration for the action."""


class ActionDto(BaseModel):
    label: Optional[str] = None
    r"""Label for the action button."""

    redirect: Optional[RedirectDto] = None
    r"""Redirect configuration for the action."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .actiondto import ActionDto, ActionDtoTypedDict
from .inappcontrolsmetadataresponsedto import (
    InAppControlsMetadataResponseDto,
    InAppControlsMetadataResponseDtoTypedDict,
)
from .redirectdto import RedirectDto, RedirectDtoTypedDict
from .resourceoriginenum import ResourceOriginEnum
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
import pydantic
from pydantic import ConfigDict
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InAppStepResponseDtoControlValuesTypedDict(TypedDict):
    r"""Control values for the in-app step"""

    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    body: NotRequired[str]
    r"""Content/body of the in-app message. Required if subject is empty."""
    subject: NotRequired[str]
    r"""Subject/title of the in-app message. Required if body is empty."""
    avatar: NotRequired[str]
    r"""URL for an avatar image. Must be a valid URL or start with / or {{ variable }}."""
    primary_action: NotRequired[ActionDtoTypedDict]
    r"""Primary action button details."""
    secondary_action: NotRequired[ActionDtoTypedDict]
    r"""Secondary action button details."""
    redirect: NotRequired[RedirectDtoTypedDict]
    r"""Redirection URL configuration for the main content click (if no actions defined/clicked).."""
    disable_output_sanitization: NotRequired[bool]
    r"""Disable sanitization of the output."""
    data: NotRequired[Dict[str, Any]]
    r"""Additional data payload for the step."""


class InAppStepResponseDtoControlValues(BaseModel):
    r"""Control values for the in-app step"""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    body: Optional[str] = None
    r"""Content/body of the in-app message. Required if subject is empty."""

    subject: Optional[str] = None
    r"""Subject/title of the in-app message. Required if body is empty."""

    avatar: Optional[str] = None
    r"""URL for an avatar image. Must be a valid URL or start with / or {{ variable }}."""

    primary_action: Annotated[
        Optional[ActionDto], pydantic.Field(alias="primaryAction")
    ] = None
    r"""Primary action button details."""

    secondary_action: Annotated[
        Optional[ActionDto], pydantic.Field(alias="secondaryAction")
    ] = None
    r"""Secondary action button details."""

    redirect: Optional[RedirectDto] = None
    r"""Redirection URL configuration for the main content click (if no actions defined/clicked).."""

    disable_output_sanitization: Annotated[
        Optional[bool], pydantic.Field(alias="disableOutputSanitization")
    ] = False
    r"""Disable sanitization of the output."""

    data: Optional[Dict[str, Any]] = None
    r"""Additional data payload for the step."""

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]


class InAppStepResponseDtoTypedDict(TypedDict):
    controls: InAppControlsMetadataResponseDtoTypedDict
    r"""Controls metadata for the in-app step"""
    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""
    step_id: str
    r"""Unique identifier of the step"""
    id: str
    r"""Database identifier of the step"""
    name: str
    r"""Name of the step"""
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    workflow_id: str
    r"""Workflow identifier"""
    workflow_database_id: str
    r"""Workflow database identifier"""
    control_values: NotRequired[InAppStepResponseDtoControlValuesTypedDict]
    r"""Control values for the in-app step"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class InAppStepResponseDto(BaseModel):
    controls: InAppControlsMetadataResponseDto
    r"""Controls metadata for the in-app step"""

    variables: Dict[str, Any]
    r"""JSON Schema for variables, follows the JSON Schema standard"""

    step_id: Annotated[str, pydantic.Field(alias="stepId")]
    r"""Unique identifier of the step"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Database identifier of the step"""

    name: str
    r"""Name of the step"""

    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow identifier"""

    workflow_database_id: Annotated[str, pydantic.Field(alias="workflowDatabaseId")]
    r"""Workflow database identifier"""

    control_values: Annotated[
        Optional[InAppStepResponseDtoControlValues],
        pydantic.Field(alias="controlValues"),
    ] = None
    r"""Control values for the in-app step"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metadto import MetaDto, MetaDtoTypedDict
from .subscriptiondto import SubscriptionDto, SubscriptionDtoTypedDict
from .subscriptionsdeleteerrordto import (
    SubscriptionsDeleteErrorDto,
    SubscriptionsDeleteErrorDtoTypedDict,
)
from novu_py.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DeleteTopicSubscriptionsResponseDtoTypedDict(TypedDict):
    data: List[SubscriptionDtoTypedDict]
    r"""The list of successfully deleted subscriptions"""
    meta: MetaDtoTypedDict
    r"""Metadata about the operation"""
    errors: NotRequired[List[SubscriptionsDeleteErrorDtoTypedDict]]
    r"""The list of errors for failed deletion attempts"""


class DeleteTopicSubscriptionsResponseDto(BaseModel):
    data: List[SubscriptionDto]
    r"""The list of successfully deleted subscriptions"""

    meta: MetaDto
    r"""Metadata about the operation"""

    errors: Optional[List[SubscriptionsDeleteErrorDto]] = None
    r"""The list of errors for failed deletion attempts"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TLSOptionsTypedDict(TypedDict):
    pass


class TLSOptions(BaseModel):
    pass


class CredentialsDtoTypedDict(TypedDict):
    api_key: NotRequired[str]
    user: NotRequired[str]
    secret_key: NotRequired[str]
    domain: NotRequired[str]
    password: NotRequired[str]
    host: NotRequired[str]
    port: NotRequired[str]
    secure: NotRequired[bool]
    region: NotRequired[str]
    account_sid: NotRequired[str]
    message_profile_id: NotRequired[str]
    token: NotRequired[str]
    from_: NotRequired[str]
    sender_name: NotRequired[str]
    project_name: NotRequired[str]
    application_id: NotRequired[str]
    client_id: NotRequired[str]
    require_tls: NotRequired[bool]
    ignore_tls: NotRequired[bool]
    tls_options: NotRequired[TLSOptionsTypedDict]
    base_url: NotRequired[str]
    webhook_url: NotRequired[str]
    redirect_url: NotRequired[str]
    hmac: NotRequired[bool]
    service_account: NotRequired[str]
    ip_pool_name: NotRequired[str]
    api_key_request_header: NotRequired[str]
    secret_key_request_header: NotRequired[str]
    id_path: NotRequired[str]
    date_path: NotRequired[str]
    api_token: NotRequired[str]
    authenticate_by_token: NotRequired[bool]
    authentication_token_key: NotRequired[str]
    instance_id: NotRequired[str]
    alert_uid: NotRequired[str]
    title: NotRequired[str]
    image_url: NotRequired[str]
    state: NotRequired[str]
    external_link: NotRequired[str]
    channel_id: NotRequired[str]
    phone_number_identification: NotRequired[str]
    access_key: NotRequired[str]
    app_sid: NotRequired[str]
    sender_id: NotRequired[str]
    tenant_id: NotRequired[str]
    app_io_base_url: NotRequired[str]


class CredentialsDto(BaseModel):
    api_key: Annotated[Optional[str], pydantic.Field(alias="apiKey")] = None

    user: Optional[str] = None

    secret_key: Annotated[Optional[str], pydantic.Field(alias="secretKey")] = None

    domain: Optional[str] = None

    password: Optional[str] = None

    host: Optional[str] = None

    port: Optional[str] = None

    secure: Optional[bool] = None

    region: Optional[str] = None

    account_sid: Annotated[Optional[str], pydantic.Field(alias="accountSid")] = None

    message_profile_id: Annotated[
        Optional[str], pydantic.Field(alias="messageProfileId")
    ] = None

    token: Optional[str] = None

    from_: Annotated[Optional[str], pydantic.Field(alias="from")] = None

    sender_name: Annotated[Optional[str], pydantic.Field(alias="senderName")] = None

    project_name: Annotated[Optional[str], pydantic.Field(alias="projectName")] = None

    application_id: Annotated[Optional[str], pydantic.Field(alias="applicationId")] = (
        None
    )

    client_id: Annotated[Optional[str], pydantic.Field(alias="clientId")] = None

    require_tls: Annotated[Optional[bool], pydantic.Field(alias="requireTls")] = None

    ignore_tls: Annotated[Optional[bool], pydantic.Field(alias="ignoreTls")] = None

    tls_options: Annotated[Optional[TLSOptions], pydantic.Field(alias="tlsOptions")] = (
        None
    )

    base_url: Annotated[Optional[str], pydantic.Field(alias="baseUrl")] = None

    webhook_url: Annotated[Optional[str], pydantic.Field(alias="webhookUrl")] = None

    redirect_url: Annotated[Optional[str], pydantic.Field(alias="redirectUrl")] = None

    hmac: Optional[bool] = None

    service_account: Annotated[
        Optional[str], pydantic.Field(alias="serviceAccount")
    ] = None

    ip_pool_name: Annotated[Optional[str], pydantic.Field(alias="ipPoolName")] = None

    api_key_request_header: Annotated[
        Optional[str], pydantic.Field(alias="apiKeyRequestHeader")
    ] = None

    secret_key_request_header: Annotated[
        Optional[str], pydantic.Field(alias="secretKeyRequestHeader")
    ] = None

    id_path: Annotated[Optional[str], pydantic.Field(alias="idPath")] = None

    date_path: Annotated[Optional[str], pydantic.Field(alias="datePath")] = None

    api_token: Annotated[Optional[str], pydantic.Field(alias="apiToken")] = None

    authenticate_by_token: Annotated[
        Optional[bool], pydantic.Field(alias="authenticateByToken")
    ] = None

    authentication_token_key: Annotated[
        Optional[str], pydantic.Field(alias="authenticationTokenKey")
    ] = None

    instance_id: Annotated[Optional[str], pydantic.Field(alias="instanceId")] = None

    alert_uid: Annotated[Optional[str], pydantic.Field(alias="alertUid")] = None

    title: Optional[str] = None

    image_url: Annotated[Optional[str], pydantic.Field(alias="imageUrl")] = None

    state: Optional[str] = None

    external_link: Annotated[Optional[str], pydantic.Field(alias="externalLink")] = None

    channel_id: Annotated[Optional[str], pydantic.Field(alias="channelId")] = None

    phone_number_identification: Annotated[
        Optional[str], pydantic.Field(alias="phoneNumberIdentification")
    ] = None

    access_key: Annotated[Optional[str], pydantic.Field(alias="accessKey")] = None

    app_sid: Annotated[Optional[str], pydantic.Field(alias="appSid")] = None

    sender_id: Annotated[Optional[str], pydantic.Field(alias="senderId")] = None

    tenant_id: Annotated[Optional[str], pydantic.Field(alias="tenantId")] = None

    app_io_base_url: Annotated[Optional[str], pydantic.Field(alias="AppIOBaseUrl")] = (
        None
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PreferenceLevelEnum(str, Enum):
    r"""The level of the preference (global or template)"""

    GLOBAL = "global"
    TEMPLATE = "template"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FailedOperationDtoTypedDict(TypedDict):
    message: NotRequired[str]
    r"""The error message associated with the failed operation."""
    subscriber_id: NotRequired[str]
    r"""The subscriber ID associated with the failed operation. This field is optional."""


class FailedOperationDto(BaseModel):
    message: Optional[str] = None
    r"""The error message associated with the failed operation."""

    subscriber_id: Annotated[Optional[str], pydantic.Field(alias="subscriberId")] = None
    r"""The subscriber ID associated with the failed operation. This field is optional."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from novu_py import models, utils
from novu_py._hooks import HookContext, SDKHooks
from novu_py.types import OptionalNullable, UNSET
from novu_py.utils import get_security_from_env
from novu_py.utils.unmarshal_json_response import unmarshal_json_response
import sys
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Mapping,
    Optional,
    TYPE_CHECKING,
    Union,
    cast,
)
import weakref

if TYPE_CHECKING:
    from novu_py.activity import Activity
    from novu_py.channel_connections import ChannelConnections
    from novu_py.channel_endpoints import ChannelEndpoints
    from novu_py.contexts import Contexts
    from novu_py.environments import Environments
    from novu_py.integrations import Integrations
    from novu_py.layouts import Layouts
    from novu_py.messages import Messages
    from novu_py.notifications import Notifications
    from novu_py.subscribers import Subscribers
    from novu_py.topics import Topics
    from novu_py.translations import Translations
    from novu_py.workflows import Workflows


class Novu(BaseSDK):
    r"""Novu API: Novu REST API. Please see https://docs.novu.co/api-reference for more details.
    https://docs.novu.co - Novu Documentation
    """

    contexts: "Contexts"
    environments: "Environments"
    r"""Environments allow you to manage different stages of your application development lifecycle. Each environment has its own set of API keys and configurations, enabling you to separate development, staging, and production workflows.
    https://docs.novu.co/platform/environments
    """
    activity: "Activity"
    layouts: "Layouts"
    r"""Layouts are reusable wrappers for your email notifications.
    https://docs.novu.co/platform/workflow/layouts
    """
    subscribers: "Subscribers"
    r"""A subscriber in Novu represents someone who should receive a message. A subscriber's profile information contains important attributes about the subscriber that will be used in messages (name, email). The subscriber object can contain other key-value pairs that can be used to further personalize your messages.
    https://docs.novu.co/subscribers/subscribers
    """
    topics: "Topics"
    r"""Topics are a way to group subscribers together so that they can be notified of events at once. A topic is identified by a custom key. This can be helpful for things like sending out marketing emails or notifying users of new features. Topics can also be used to send notifications to the subscribers who have been grouped together based on their interests, location, activities and much more.
    https://docs.novu.co/subscribers/topics
    """
    translations: "Translations"
    r"""Used to localize your notifications to different languages.
    https://docs.novu.co/platform/workflow/translations
    """
    workflows: "Workflows"
    r"""All notifications are sent via a workflow. Each workflow acts as a container for the logic and blueprint that are associated with a type of notification in your system.
    https://docs.novu.co/workflows
    """
    channel_connections: "ChannelConnections"
    channel_endpoints: "ChannelEndpoints"
    integrations: "Integrations"
    r"""With the help of the Integration Store, you can easily integrate your favorite delivery provider. During the runtime of the API, the Integrations Store is responsible for storing the configurations of all the providers.
    https://docs.novu.co/platform/integrations/overview
    """
    messages: "Messages"
    r"""A message in Novu represents a notification delivered to a recipient on a particular channel. Messages contain information about the request that triggered its delivery, a view of the data sent to the recipient, and a timeline of its lifecycle events. Learn more about messages.
    https://docs.novu.co/workflows/messages
    """
    notifications: "Notifications"
    _sub_sdk_map = {
        "contexts": ("novu_py.contexts", "Contexts"),
        "environments": ("novu_py.environments", "Environments"),
        "activity": ("novu_py.activity", "Activity"),
        "layouts": ("novu_py.layouts", "Layouts"),
        "subscribers": ("novu_py.subscribers", "Subscribers"),
        "topics": ("novu_py.topics", "Topics"),
        "translations": ("novu_py.translations", "Translations"),
        "workflows": ("novu_py.workflows", "Workflows"),
        "channel_connections": ("novu_py.channel_connections", "ChannelConnections"),
        "channel_endpoints": ("novu_py.channel_endpoints", "ChannelEndpoints"),
        "integrations": ("novu_py.integrations", "Integrations"),
        "messages": ("novu_py.messages", "Messages"),
        "notifications": ("novu_py.notifications", "Notifications"),
    }

    def __init__(
        self,
        secret_key: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param secret_key: The secret_key required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(secret_key):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(secret_key=secret_key())
        else:
            security = models.Security(secret_key=secret_key)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None

    def trigger(
        self,
        *,
        trigger_event_request_dto: Union[
            models.TriggerEventRequestDto, models.TriggerEventRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerTriggerResponse:
        r"""Trigger event

        Trigger event is the main (and only) way to send notifications to subscribers. The trigger identifier is used to match the particular workflow associated with it. Maximum number of recipients can be 100. Additional information can be passed according the body interface below.
        To prevent duplicate triggers, you can optionally pass a **transactionId** in the request body. If the same **transactionId** is used again, the trigger will be ignored. The retention period depends on your billing tier.

        :param trigger_event_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerTriggerRequest(
            idempotency_key=idempotency_key,
            trigger_event_request_dto=utils.get_pydantic_model(
                trigger_event_request_dto, models.TriggerEventRequestDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/events/trigger",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trigger_event_request_dto,
                False,
                False,
                "json",
                models.TriggerEventRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_trigger",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.EventsControllerTriggerResponse(
                result=unmarshal_json_response(
                    models.TriggerEventResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def trigger_async(
        self,
        *,
        trigger_event_request_dto: Union[
            models.TriggerEventRequestDto, models.TriggerEventRequestDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerTriggerResponse:
        r"""Trigger event

        Trigger event is the main (and only) way to send notifications to subscribers. The trigger identifier is used to match the particular workflow associated with it. Maximum number of recipients can be 100. Additional information can be passed according the body interface below.
        To prevent duplicate triggers, you can optionally pass a **transactionId** in the request body. If the same **transactionId** is used again, the trigger will be ignored. The retention period depends on your billing tier.

        :param trigger_event_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerTriggerRequest(
            idempotency_key=idempotency_key,
            trigger_event_request_dto=utils.get_pydantic_model(
                trigger_event_request_dto, models.TriggerEventRequestDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/events/trigger",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trigger_event_request_dto,
                False,
                False,
                "json",
                models.TriggerEventRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_trigger",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.EventsControllerTriggerResponse(
                result=unmarshal_json_response(
                    models.TriggerEventResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def cancel(
        self,
        *,
        transaction_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerCancelResponse:
        r"""Cancel triggered event


        Using a previously generated transactionId during the event trigger,
        will cancel any active or pending workflows. This is useful to cancel active digests, delays etc...


        :param transaction_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerCancelRequest(
            transaction_id=transaction_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/events/trigger/{transactionId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_cancel",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EventsControllerCancelResponse(
                result=unmarshal_json_response(bool, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def cancel_async(
        self,
        *,
        transaction_id: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerCancelResponse:
        r"""Cancel triggered event


        Using a previously generated transactionId during the event trigger,
        will cancel any active or pending workflows. This is useful to cancel active digests, delays etc...


        :param transaction_id:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerCancelRequest(
            transaction_id=transaction_id,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/events/trigger/{transactionId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_cancel",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EventsControllerCancelResponse(
                result=unmarshal_json_response(bool, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def trigger_broadcast(
        self,
        *,
        trigger_event_to_all_request_dto: Union[
            models.TriggerEventToAllRequestDto,
            models.TriggerEventToAllRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerBroadcastEventToAllResponse:
        r"""Broadcast event to all

        Trigger a broadcast event to all existing subscribers, could be used to send announcements, etc.

        In the future could be used to trigger events to a subset of subscribers based on defined filters.

        :param trigger_event_to_all_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerBroadcastEventToAllRequest(
            idempotency_key=idempotency_key,
            trigger_event_to_all_request_dto=utils.get_pydantic_model(
                trigger_event_to_all_request_dto, models.TriggerEventToAllRequestDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/events/trigger/broadcast",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trigger_event_to_all_request_dto,
                False,
                False,
                "json",
                models.TriggerEventToAllRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_broadcastEventToAll",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return models.EventsControllerBroadcastEventToAllResponse(
                result=unmarshal_json_response(
                    models.TriggerEventResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def trigger_broadcast_async(
        self,
        *,
        trigger_event_to_all_request_dto: Union[
            models.TriggerEventToAllRequestDto,
            models.TriggerEventToAllRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerBroadcastEventToAllResponse:
        r"""Broadcast event to all

        Trigger a broadcast event to all existing subscribers, could be used to send announcements, etc.

        In the future could be used to trigger events to a subset of subscribers based on defined filters.

        :param trigger_event_to_all_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerBroadcastEventToAllRequest(
            idempotency_key=idempotency_key,
            trigger_event_to_all_request_dto=utils.get_pydantic_model(
                trigger_event_to_all_request_dto, models.TriggerEventToAllRequestDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/events/trigger/broadcast",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trigger_event_to_all_request_dto,
                False,
                False,
                "json",
                models.TriggerEventToAllRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_broadcastEventToAll",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, ["200", "201"], "application/json"):
            return models.EventsControllerBroadcastEventToAllResponse(
                result=unmarshal_json_response(
                    models.TriggerEventResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def trigger_bulk(
        self,
        *,
        bulk_trigger_event_dto: Union[
            models.BulkTriggerEventDto, models.BulkTriggerEventDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerTriggerBulkResponse:
        r"""Bulk trigger event


        Using this endpoint you can trigger multiple events at once, to avoid multiple calls to the API.
        The bulk API is limited to 100 events per request.


        :param bulk_trigger_event_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerTriggerBulkRequest(
            idempotency_key=idempotency_key,
            bulk_trigger_event_dto=utils.get_pydantic_model(
                bulk_trigger_event_dto, models.BulkTriggerEventDto
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/events/trigger/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.bulk_trigger_event_dto,
                False,
                False,
                "json",
                models.BulkTriggerEventDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_triggerBulk",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.EventsControllerTriggerBulkResponse(
                result=unmarshal_json_response(
                    List[models.TriggerEventResponseDto], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def trigger_bulk_async(
        self,
        *,
        bulk_trigger_event_dto: Union[
            models.BulkTriggerEventDto, models.BulkTriggerEventDtoTypedDict
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsControllerTriggerBulkResponse:
        r"""Bulk trigger event


        Using this endpoint you can trigger multiple events at once, to avoid multiple calls to the API.
        The bulk API is limited to 100 events per request.


        :param bulk_trigger_event_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsControllerTriggerBulkRequest(
            idempotency_key=idempotency_key,
            bulk_trigger_event_dto=utils.get_pydantic_model(
                bulk_trigger_event_dto, models.BulkTriggerEventDto
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/events/trigger/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.bulk_trigger_event_dto,
                False,
                False,
                "json",
                models.BulkTriggerEventDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="EventsController_triggerBulk",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.EventsControllerTriggerBulkResponse(
                result=unmarshal_json_response(
                    List[models.TriggerEventResponseDto], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PayloadValidationExceptionDtoData, http_res
            )
            raise models.PayloadValidationExceptionDto(response_data, http_res)
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ImportMasterJSONResponseDtoTypedDict(TypedDict):
    success: bool
    r"""Overall success status of the import operation"""
    message: str
    r"""Human-readable message describing the import result"""
    successful: NotRequired[List[str]]
    r"""List of resource IDs that were successfully imported"""
    failed: NotRequired[List[str]]
    r"""List of resource IDs that failed to import"""


class ImportMasterJSONResponseDto(BaseModel):
    success: bool
    r"""Overall success status of the import operation"""

    message: str
    r"""Human-readable message describing the import result"""

    successful: Optional[List[str]] = None
    r"""List of resource IDs that were successfully imported"""

    failed: Optional[List[str]] = None
    r"""List of resource IDs that failed to import"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activitynotificationresponsedto import (
    ActivityNotificationResponseDto,
    ActivityNotificationResponseDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class ActivitiesResponseDtoTypedDict(TypedDict):
    has_more: bool
    r"""Indicates if there are more activities in the result set"""
    data: List[ActivityNotificationResponseDtoTypedDict]
    r"""Array of activity notifications"""
    page_size: float
    r"""Page size of the activities"""
    page: float
    r"""Current page of the activities"""


class ActivitiesResponseDto(BaseModel):
    has_more: Annotated[bool, pydantic.Field(alias="hasMore")]
    r"""Indicates if there are more activities in the result set"""

    data: List[ActivityNotificationResponseDto]
    r"""Array of activity notifications"""

    page_size: Annotated[float, pydantic.Field(alias="pageSize")]
    r"""Page size of the activities"""

    page: float
    r"""Current page of the activities"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createlayoutdto import CreateLayoutDto, CreateLayoutDtoTypedDict
from .layoutresponsedto import LayoutResponseDto, LayoutResponseDtoTypedDict
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LayoutsControllerCreateRequestTypedDict(TypedDict):
    create_layout_dto: CreateLayoutDtoTypedDict
    r"""Layout creation details"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class LayoutsControllerCreateRequest(BaseModel):
    create_layout_dto: Annotated[
        CreateLayoutDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Layout creation details"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class LayoutsControllerCreateResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: LayoutResponseDtoTypedDict


class LayoutsControllerCreateResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: LayoutResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class Operator(str, Enum):
    LARGER = "LARGER"
    SMALLER = "SMALLER"
    LARGER_EQUAL = "LARGER_EQUAL"
    SMALLER_EQUAL = "SMALLER_EQUAL"
    EQUAL = "EQUAL"
    NOT_EQUAL = "NOT_EQUAL"
    ALL_IN = "ALL_IN"
    ANY_IN = "ANY_IN"
    NOT_IN = "NOT_IN"
    BETWEEN = "BETWEEN"
    NOT_BETWEEN = "NOT_BETWEEN"
    LIKE = "LIKE"
    NOT_LIKE = "NOT_LIKE"
    IN = "IN"


class On(str, Enum):
    SUBSCRIBER = "subscriber"
    PAYLOAD = "payload"


class FieldFilterPartDtoTypedDict(TypedDict):
    field: str
    value: str
    operator: Operator
    on: On


class FieldFilterPartDto(BaseModel):
    field: str

    value: str

    operator: Operator

    on: On



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowPreferenceRequestDtoTypedDict(TypedDict):
    workflow_id: str
    r"""The workflow identifier"""
    enabled: NotRequired[bool]
    r"""Whether the preference is enabled. Used when condition is not provided."""
    condition: NotRequired[Dict[str, Any]]
    r"""Optional condition using JSON Logic rules"""


class WorkflowPreferenceRequestDto(BaseModel):
    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""The workflow identifier"""

    enabled: Optional[bool] = None
    r"""Whether the preference is enabled. Used when condition is not provided."""

    condition: Optional[Dict[str, Any]] = None
    r"""Optional condition using JSON Logic rules"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriberPreferencesWorkflowInfoDtoTypedDict(TypedDict):
    slug: str
    r"""Workflow slug"""
    identifier: str
    r"""Unique identifier of the workflow"""
    name: str
    r"""Display name of the workflow"""
    updated_at: NotRequired[str]
    r"""last updated date"""


class SubscriberPreferencesWorkflowInfoDto(BaseModel):
    slug: str
    r"""Workflow slug"""

    identifier: str
    r"""Unique identifier of the workflow"""

    name: str
    r"""Display name of the workflow"""

    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""last updated date"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AuthDtoTypedDict(TypedDict):
    access_token: str


class AuthDto(BaseModel):
    access_token: Annotated[str, pydantic.Field(alias="accessToken")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationGroupTypedDict(TypedDict):
    name: str
    environment_id: str
    organization_id: str
    id: NotRequired[str]
    parent_id: NotRequired[str]


class NotificationGroup(BaseModel):
    name: str

    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createupdatetopicrequestdto import (
    CreateUpdateTopicRequestDto,
    CreateUpdateTopicRequestDtoTypedDict,
)
from .topicresponsedto import TopicResponseDto, TopicResponseDtoTypedDict
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicsControllerUpsertTopicRequestTypedDict(TypedDict):
    create_update_topic_request_dto: CreateUpdateTopicRequestDtoTypedDict
    fail_if_exists: NotRequired[bool]
    r"""If true, the request will fail if a topic with the same key already exists"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class TopicsControllerUpsertTopicRequest(BaseModel):
    create_update_topic_request_dto: Annotated[
        CreateUpdateTopicRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    fail_if_exists: Annotated[
        Optional[bool],
        pydantic.Field(alias="failIfExists"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""If true, the request will fail if a topic with the same key already exists"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class TopicsControllerUpsertTopicResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: TopicResponseDtoTypedDict


class TopicsControllerUpsertTopicResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: TopicResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class Type(str, Enum):
    r"""Type of the delay. Currently only 'regular' is supported by the schema."""

    REGULAR = "regular"
    TIMED = "timed"


class Unit(str, Enum):
    r"""Unit of time for the delay amount."""

    SECONDS = "seconds"
    MINUTES = "minutes"
    HOURS = "hours"
    DAYS = "days"
    WEEKS = "weeks"
    MONTHS = "months"


class DelayControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    type: NotRequired[Type]
    r"""Type of the delay. Currently only 'regular' is supported by the schema."""
    amount: NotRequired[float]
    r"""Amount of time to delay."""
    unit: NotRequired[Unit]
    r"""Unit of time for the delay amount."""
    cron: NotRequired[str]
    r"""Cron expression for the delay. Min length 1."""


class DelayControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    type: Optional[Type] = Type.REGULAR
    r"""Type of the delay. Currently only 'regular' is supported by the schema."""

    amount: Optional[float] = None
    r"""Amount of time to delay."""

    unit: Optional[Unit] = None
    r"""Unit of time for the delay amount."""

    cron: Optional[str] = None
    r"""Cron expression for the delay. Min length 1."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from novu_py.models import NovuError
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class TopicResponseDtoErrorData(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The identifier of the topic"""
    key: str
    r"""The unique key of the topic"""
    name: Optional[str] = None
    r"""The name of the topic"""
    created_at: Annotated[Optional[str], pydantic.Field(alias="createdAt")] = None
    r"""The date the topic was created"""
    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""The date the topic was last updated"""


@dataclass(unsafe_hash=True)
class TopicResponseDtoError(NovuError):
    data: TopicResponseDtoErrorData = field(hash=False)

    def __init__(
        self,
        data: TopicResponseDtoErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WorkflowInfoDtoTypedDict(TypedDict):
    name: str
    r"""The name of the workflow"""
    workflow_id: str
    r"""The unique identifier of the workflow"""


class WorkflowInfoDto(BaseModel):
    name: str
    r"""The name of the workflow"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""The unique identifier of the workflow"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ResourceOriginEnum(str, Enum):
    r"""Origin of the layout"""

    NOVU_CLOUD = "novu-cloud"
    NOVU_CLOUD_V1 = "novu-cloud-v1"
    EXTERNAL = "external"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UIComponentEnum(str, Enum):
    r"""Component type for the UI Schema Property"""

    EMAIL_EDITOR_SELECT = "EMAIL_EDITOR_SELECT"
    LAYOUT_SELECT = "LAYOUT_SELECT"
    BLOCK_EDITOR = "BLOCK_EDITOR"
    EMAIL_BODY = "EMAIL_BODY"
    TEXT_FULL_LINE = "TEXT_FULL_LINE"
    TEXT_INLINE_LABEL = "TEXT_INLINE_LABEL"
    IN_APP_BODY = "IN_APP_BODY"
    IN_APP_AVATAR = "IN_APP_AVATAR"
    IN_APP_PRIMARY_SUBJECT = "IN_APP_PRIMARY_SUBJECT"
    IN_APP_BUTTON_DROPDOWN = "IN_APP_BUTTON_DROPDOWN"
    IN_APP_DISABLE_SANITIZATION_SWITCH = "IN_APP_DISABLE_SANITIZATION_SWITCH"
    DISABLE_SANITIZATION_SWITCH = "DISABLE_SANITIZATION_SWITCH"
    URL_TEXT_BOX = "URL_TEXT_BOX"
    DIGEST_AMOUNT = "DIGEST_AMOUNT"
    DIGEST_UNIT = "DIGEST_UNIT"
    DIGEST_TYPE = "DIGEST_TYPE"
    DIGEST_KEY = "DIGEST_KEY"
    DIGEST_CRON = "DIGEST_CRON"
    DELAY_AMOUNT = "DELAY_AMOUNT"
    DELAY_UNIT = "DELAY_UNIT"
    DELAY_TYPE = "DELAY_TYPE"
    DELAY_CRON = "DELAY_CRON"
    DELAY_DYNAMIC_KEY = "DELAY_DYNAMIC_KEY"
    THROTTLE_TYPE = "THROTTLE_TYPE"
    THROTTLE_WINDOW = "THROTTLE_WINDOW"
    THROTTLE_UNIT = "THROTTLE_UNIT"
    THROTTLE_DYNAMIC_KEY = "THROTTLE_DYNAMIC_KEY"
    THROTTLE_THRESHOLD = "THROTTLE_THRESHOLD"
    THROTTLE_KEY = "THROTTLE_KEY"
    EXTEND_TO_SCHEDULE = "EXTEND_TO_SCHEDULE"
    SMS_BODY = "SMS_BODY"
    CHAT_BODY = "CHAT_BODY"
    PUSH_BODY = "PUSH_BODY"
    PUSH_SUBJECT = "PUSH_SUBJECT"
    QUERY_EDITOR = "QUERY_EDITOR"
    DATA = "DATA"
    LAYOUT_EMAIL = "LAYOUT_EMAIL"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class TimeRangeDtoTypedDict(TypedDict):
    start: str
    r"""Start time"""
    end: str
    r"""End time"""


class TimeRangeDto(BaseModel):
    start: str
    r"""Start time"""

    end: str
    r"""End time"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .patchpreferencechannelsdto import (
    PatchPreferenceChannelsDto,
    PatchPreferenceChannelsDtoTypedDict,
)
from .scheduledto import ScheduleDto, ScheduleDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PatchSubscriberPreferencesDtoTypedDict(TypedDict):
    channels: NotRequired[PatchPreferenceChannelsDtoTypedDict]
    r"""Channel-specific preference settings"""
    workflow_id: NotRequired[str]
    r"""Workflow internal _id, identifier or slug. If provided, update workflow specific preferences, otherwise update global preferences"""
    schedule: NotRequired[ScheduleDtoTypedDict]
    r"""Subscriber schedule"""


class PatchSubscriberPreferencesDto(BaseModel):
    channels: Optional[PatchPreferenceChannelsDto] = None
    r"""Channel-specific preference settings"""

    workflow_id: Annotated[Optional[str], pydantic.Field(alias="workflowId")] = None
    r"""Workflow internal _id, identifier or slug. If provided, update workflow specific preferences, otherwise update global preferences"""

    schedule: Optional[ScheduleDto] = None
    r"""Subscriber schedule"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


Value5TypedDict = TypeAliasType(
    "Value5TypedDict", Union[str, float, bool, Dict[str, Any]]
)


Value5 = TypeAliasType("Value5", Union[str, float, bool, Dict[str, Any]])


class Value4TypedDict(TypedDict):
    pass


class Value4(BaseModel):
    pass


PayloadValidationErrorDtoValueTypedDict = TypeAliasType(
    "PayloadValidationErrorDtoValueTypedDict",
    Union[Value4TypedDict, str, float, bool, List[Nullable[Value5TypedDict]]],
)
r"""The actual value that failed validation"""


PayloadValidationErrorDtoValue = TypeAliasType(
    "PayloadValidationErrorDtoValue",
    Union[Value4, str, float, bool, List[Nullable[Value5]]],
)
r"""The actual value that failed validation"""


class PayloadValidationErrorDtoTypedDict(TypedDict):
    field: str
    r"""Field path that failed validation"""
    message: str
    r"""Validation error message"""
    value: NotRequired[Nullable[PayloadValidationErrorDtoValueTypedDict]]
    r"""The actual value that failed validation"""
    schema_path: NotRequired[str]
    r"""JSON Schema path where the validation failed"""


class PayloadValidationErrorDto(BaseModel):
    field: str
    r"""Field path that failed validation"""

    message: str
    r"""Validation error message"""

    value: OptionalNullable[PayloadValidationErrorDtoValue] = UNSET
    r"""The actual value that failed validation"""

    schema_path: Annotated[Optional[str], pydantic.Field(alias="schemaPath")] = None
    r"""JSON Schema path where the validation failed"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["value", "schemaPath"]
        nullable_fields = ["value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .triggerrecipientstypeenum import TriggerRecipientsTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicPayloadDtoTypedDict(TypedDict):
    topic_key: str
    type: TriggerRecipientsTypeEnum
    exclude: NotRequired[List[str]]
    r"""Optional array of subscriber IDs to exclude from the topic trigger"""


class TopicPayloadDto(BaseModel):
    topic_key: Annotated[str, pydantic.Field(alias="topicKey")]

    type: TriggerRecipientsTypeEnum

    exclude: Optional[List[str]] = None
    r"""Optional array of subscriber IDs to exclude from the topic trigger"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationgroup import NotificationGroup, NotificationGroupTypedDict
from .notificationstepdto import NotificationStepDto, NotificationStepDtoTypedDict
from .notificationtrigger import NotificationTrigger, NotificationTriggerTypedDict
from .subscriberpreferencechannels import (
    SubscriberPreferenceChannels,
    SubscriberPreferenceChannelsTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowResponseDataTypedDict(TypedDict):
    pass


class WorkflowResponseData(BaseModel):
    pass


class WorkflowIntegrationStatusTypedDict(TypedDict):
    pass


class WorkflowIntegrationStatus(BaseModel):
    pass


class WorkflowResponseTypedDict(TypedDict):
    name: str
    description: str
    active: bool
    draft: bool
    preference_settings: SubscriberPreferenceChannelsTypedDict
    critical: bool
    tags: List[str]
    steps: List[NotificationStepDtoTypedDict]
    organization_id: str
    creator_id: str
    environment_id: str
    triggers: List[NotificationTriggerTypedDict]
    notification_group_id: str
    deleted: bool
    deleted_at: str
    deleted_by: str
    id: NotRequired[str]
    parent_id: NotRequired[str]
    notification_group: NotRequired[NotificationGroupTypedDict]
    data: NotRequired[WorkflowResponseDataTypedDict]
    workflow_integration_status: NotRequired[WorkflowIntegrationStatusTypedDict]


class WorkflowResponse(BaseModel):
    name: str

    description: str

    active: bool

    draft: bool

    preference_settings: Annotated[
        SubscriberPreferenceChannels, pydantic.Field(alias="preferenceSettings")
    ]

    critical: bool

    tags: List[str]

    steps: List[NotificationStepDto]

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]

    creator_id: Annotated[str, pydantic.Field(alias="_creatorId")]

    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]

    triggers: List[NotificationTrigger]

    notification_group_id: Annotated[str, pydantic.Field(alias="_notificationGroupId")]

    deleted: bool

    deleted_at: Annotated[str, pydantic.Field(alias="deletedAt")]

    deleted_by: Annotated[str, pydantic.Field(alias="deletedBy")]

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None

    notification_group: Annotated[
        Optional[NotificationGroup], pydantic.Field(alias="notificationGroup")
    ] = None

    data: Optional[WorkflowResponseData] = None

    workflow_integration_status: Annotated[
        Optional[WorkflowIntegrationStatus],
        pydantic.Field(alias="workflowIntegrationStatus"),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .credentialsdto import CredentialsDto, CredentialsDtoTypedDict
from .stepfilterdto import StepFilterDto, StepFilterDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateIntegrationRequestDtoConfigurationsTypedDict(TypedDict):
    r"""Configurations for the integration"""


class UpdateIntegrationRequestDtoConfigurations(BaseModel):
    r"""Configurations for the integration"""


class UpdateIntegrationRequestDtoTypedDict(TypedDict):
    name: NotRequired[str]
    identifier: NotRequired[str]
    environment_id: NotRequired[str]
    active: NotRequired[bool]
    r"""If the integration is active the validation on the credentials field will run"""
    credentials: NotRequired[CredentialsDtoTypedDict]
    check: NotRequired[bool]
    conditions: NotRequired[List[StepFilterDtoTypedDict]]
    configurations: NotRequired[UpdateIntegrationRequestDtoConfigurationsTypedDict]
    r"""Configurations for the integration"""


class UpdateIntegrationRequestDto(BaseModel):
    name: Optional[str] = None

    identifier: Optional[str] = None

    environment_id: Annotated[Optional[str], pydantic.Field(alias="_environmentId")] = (
        None
    )

    active: Optional[bool] = None
    r"""If the integration is active the validation on the credentials field will run"""

    credentials: Optional[CredentialsDto] = None

    check: Optional[bool] = None

    conditions: Optional[List[StepFilterDto]] = None

    configurations: Optional[UpdateIntegrationRequestDtoConfigurations] = None
    r"""Configurations for the integration"""
